/*

Qanac, the Quick Analytic Accounting
Copyright (C) 2006 Stephane Magnenat <stephane at magnenat dot net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "budgetdialog.h"
#include <QDoubleSpinBox>
#include <QComboBox>
#include <QTableWidget>
#include <QPushButton>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QScrollArea>
#include <QDomDocument>
#include <QDomElement>

BudgetDialog::BudgetDialog(const QTableWidget *accounts, QDomDocument *document) :
	domDocument(document)
{
	source = new QComboBox;
	QVBoxLayout *scrollLayout = new QVBoxLayout;
	amounts.resize(accounts->rowCount());
	labels.resize(accounts->rowCount());
	
	QDomElement budgetElement = domDocument->documentElement().firstChildElement("budget");
	for (int i = 0; i < accounts->rowCount(); i++)
	{
		const QString &accountName = accounts->item(i, 0)->text();
		source->addItem(accountName);
		
		amounts[i] = new QDoubleSpinBox;
		amounts[i]->setRange(0, 1000000000);
		labels[i] = new QLabel;
		labels[i]->setText(accountName);
		QHBoxLayout *accountLayout = new QHBoxLayout;
		accountLayout->addWidget(labels[i]);
		accountLayout->addWidget(amounts[i]);
		scrollLayout->addLayout(accountLayout);
		
		// if this is source, fill source from DOM
		if (budgetElement.hasAttribute("source") && (budgetElement.attribute("source") == accountName))
		{
			source->setCurrentIndex(i);
			amounts[i]->setEnabled(false);
		}
		
		// fill amount from DOM
		QDomElement n = budgetElement.firstChildElement("transfer");
		while (!n.isNull())
		{
			if (n.attribute("destination") == accountName)
				amounts[i]->setValue(n.attribute("amount").toInt());
			n = n.nextSiblingElement ("transfer");
		}
	}
	if (!budgetElement.hasAttribute("source"))
	{
		if (amounts.count() > 0)
			amounts[0]->setEnabled(false);
	}
	
	connect(source, SIGNAL(currentIndexChanged( const QString &)), SLOT(sourceChanged(const QString &)));
	
	QHBoxLayout *sourceLayout = new QHBoxLayout;
	sourceLayout->addWidget(new QLabel(tr("Source account:")));
	sourceLayout->addWidget(source);
	
	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->addLayout(sourceLayout);
	QWidget *scollContent = new QWidget;
	scollContent->setLayout(scrollLayout);
	QScrollArea *scrollArea = new QScrollArea;
	scrollArea->setWidget(scollContent);
	layout->addWidget(scrollArea);
	
	QPushButton *ok = new QPushButton(tr("Ok"));
	connect(ok, SIGNAL(clicked()), SLOT(budgetAccepted()));
	layout->addWidget(ok);
	
	setWindowTitle(tr("Budget setup"));
}

void BudgetDialog::budgetAccepted()
{
	domDocument->documentElement().removeChild(domDocument->documentElement().firstChildElement("budget"));
	QDomElement budgetElement = domDocument->createElement("budget");
	domDocument->documentElement().appendChild(budgetElement);
	budgetElement.setAttribute("source", source->currentText());
	
	for (int i = 0; i < amounts.count(); i++)
	{
		QDomElement transferElement = domDocument->createElement("transfer");
		transferElement.setAttribute("destination", labels[i]->text());
		transferElement.setAttribute("amount", amounts[i]->value());
		budgetElement.appendChild(transferElement);
	}
	
	accept();
}

void BudgetDialog::sourceChanged(const QString &newSource)
{
	for (int i = 0; i < amounts.count(); i++)
	{
		if (labels[i]->text() == newSource)
			amounts[i]->setEnabled(false);
		else
			amounts[i]->setEnabled(true);
	}
}


