/*

Qanac, the Quick Analytic Accounting
Copyright (C) 2006 Stephane Magnenat <stephane at magnenat dot net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "mainwindow.h"
#include "transferdialog.h"
#include "budgetdialog.h"

#include <QListWidget>
#include <QTableWidget>
#include <QPushButton>
#include <QLabel>
#include <QToolBar>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QStatusBar>
#include <QFile>
#include <QFileDialog>
#include <QMessageBox>
#include <QCloseEvent>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QDate>
#include <QHeaderView>
#include <QSplitter>
#include <QCheckBox>
#include <QApplication>
#include <QtDebug>


MainWindow::MainWindow() :
	domDocument("qanac")
{
	domDocument.appendChild(domDocument.createElement("qanac"));
	
	accountList = new QTableWidget;
	accountList->setShowGrid(false);
	accountList->setSelectionMode(QAbstractItemView::SingleSelection);
	accountList->setSelectionBehavior(QAbstractItemView::SelectRows);
	accountList->setSortingEnabled(false);
	accountList->setDragEnabled(false);
	accountList->setEditTriggers(QAbstractItemView::DoubleClicked | QAbstractItemView::SelectedClicked);
	accountList->setColumnCount(2);
	accountList->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
	accountList->horizontalHeader()->hide();
	accountList->verticalHeader()->hide();
	connect(accountList, SIGNAL(currentCellChanged(int,int,int,int)), this, SLOT(fillTransferList(int)));
	newAccountButton = new QPushButton(tr("New"));
	newAccountButton->setStatusTip(tr("Create a new account"));
	connect(newAccountButton, SIGNAL(clicked()), this, SLOT(newAccount()));
	deleteAccountButton = new QPushButton(tr("Delete"));
	deleteAccountButton->setStatusTip(tr("Delete an existing account"));
	connect(deleteAccountButton, SIGNAL(clicked()), this, SLOT(deleteAccount()));
	transferList = new QTableWidget;
	transferList->setShowGrid(false);
	transferList->setSelectionMode(QAbstractItemView::SingleSelection);
	transferList->setSelectionBehavior(QAbstractItemView::SelectRows);
	transferList->setSortingEnabled(false);
	transferList->setDragEnabled(false);
	transferList->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
	transferList->horizontalHeader()->hide();
	transferList->verticalHeader()->hide();
	newTransferButton = new QPushButton(tr("New"));
	newTransferButton->setStatusTip(tr("Create a new transfer"));
	connect(newTransferButton, SIGNAL(clicked()), this, SLOT(newTransfer()));
	deleteTransferButton = new QPushButton(tr("Delete"));
	deleteTransferButton->setStatusTip(tr("Delete an existing transfer"));
	connect(deleteTransferButton, SIGNAL(clicked()), this, SLOT(deleteTransfer()));
	
	QHBoxLayout *leftAlmostBottomLayout = new QHBoxLayout;
	leftAlmostBottomLayout->addWidget(new QLabel(tr("Total")));
	balance = new QLabel("0");
	leftAlmostBottomLayout->addWidget(balance);
	QHBoxLayout *leftBottomLayout = new QHBoxLayout;
	leftBottomLayout->addWidget(newAccountButton);
	leftBottomLayout->addWidget(deleteAccountButton);
	QVBoxLayout *leftLayout = new QVBoxLayout;
	leftLayout->addWidget(new QLabel("Accounts"));
	leftLayout->addWidget(accountList);
	leftLayout->addLayout(leftAlmostBottomLayout);
	leftLayout->addLayout(leftBottomLayout);
	QHBoxLayout *rightBottomLayout = new QHBoxLayout;
	rightBottomLayout->addWidget(newTransferButton);
	rightBottomLayout->addWidget(deleteTransferButton);
	QVBoxLayout *rightLayout = new QVBoxLayout;
	rightLayout->addWidget(new QLabel("Transfer"));
	rightLayout->addWidget(transferList);
	rightLayout->addLayout(rightBottomLayout);
	QWidget *leftWidget = new QWidget;
	leftWidget->setLayout(leftLayout);
	QWidget *rightWidget = new QWidget;
	rightWidget->setLayout(rightLayout);
	QSplitter *splitter = new QSplitter;
	splitter->addWidget(leftWidget);
	splitter->addWidget(rightWidget);
	splitter->setStretchFactor(0, 6);
	splitter->setStretchFactor(1, 11);
	setCentralWidget(splitter);
	
	createActions();
	createMenus();
	createToolBars();
	createStatusBar();
	
	setWindowTitle(tr("Quick Analytic Accounting"));
	resize(620, 440);
	unsavedChanges = false;
}

void MainWindow::createActions()
{
	newAct = new QAction(tr("&New"), this);
	newAct->setShortcut(tr("Ctrl+N"));
	newAct->setStatusTip(tr("Create a new file"));
	connect(newAct, SIGNAL(triggered()), SLOT(newFile()));
	openAct = new QAction(tr("&Open"), this);
	openAct->setShortcut(tr("Ctrl+O"));
	openAct->setStatusTip(tr("Open an existing file"));
	connect(openAct, SIGNAL(triggered()), SLOT(load()));
	saveAct = new QAction(tr("&Save"), this);
	saveAct->setShortcut(tr("Ctrl+S"));
	saveAct->setStatusTip(tr("Save this file"));
	connect(saveAct, SIGNAL(triggered()), SLOT(save()));
	quitAct = new QAction(tr("&Quit"), this);
	quitAct->setShortcut(tr("Ctrl+Q"));
	quitAct->setStatusTip(tr("Quit Qanac"));
	connect(quitAct, SIGNAL(triggered()), SLOT(close()));
	
	setupBudgetAct = new QAction(tr("Se&tup budget"), this);
	setupBudgetAct->setShortcut(tr("Ctrl+T"));
	setupBudgetAct->setStatusTip(tr("Setup money distribution into accounts"));
	connect(setupBudgetAct, SIGNAL(triggered()), SLOT(setupBudget()));
	distributeBudgetAct = new QAction(tr("&Distribute budget"), this);
	distributeBudgetAct->setShortcut(tr("Ctrl+D"));
	distributeBudgetAct->setStatusTip(tr("Distribute budget as previously setuped"));
	connect(distributeBudgetAct, SIGNAL(triggered()), SLOT(distributeBudget()));
	
	aboutAct = new QAction(tr("&About Qanac"), this);
	connect(aboutAct, SIGNAL(triggered()), SLOT(about()));
	aboutQtAct = new QAction(tr("About &Qt"), this);
	connect(aboutQtAct, SIGNAL(triggered()), QApplication::instance(), SLOT(aboutQt()));
}

void MainWindow::createMenus()
{
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(newAct);
	fileMenu->addAction(openAct);
	fileMenu->addAction(saveAct);
	fileMenu->addSeparator();
	fileMenu->addAction(quitAct);

	budgetMenu = menuBar()->addMenu(tr("&Budget"));
	budgetMenu->addAction(setupBudgetAct);
	budgetMenu->addAction(distributeBudgetAct);

	menuBar()->addSeparator();

	helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(aboutAct);
	helpMenu->addAction(aboutQtAct);
}

void MainWindow::createToolBars()
{
	budgetToolBar = addToolBar(tr("file"));
	budgetToolBar->addAction(newAct);
	budgetToolBar->addAction(openAct);
	budgetToolBar->addAction(saveAct);
	
	budgetToolBar = addToolBar(tr("budget"));
	budgetToolBar->addAction(setupBudgetAct);
	budgetToolBar->addAction(distributeBudgetAct);
}

void MainWindow::createStatusBar()
{
	statusBar()->showMessage(tr("Ready"));
}


bool MainWindow::loadFile(const QString &filename)
{
	QFile file(filename);
	if (file.open(QIODevice::ReadOnly))
	{
		QString errorStr;
		int errorLine;
		int errorColumn;
		
		saveFileName = filename;
	
		if (!domDocument.setContent(&file, false, &errorStr, &errorLine, &errorColumn))
		{
			QMessageBox::information(window(), "Qanac",
									tr("Parse error at line %1, column %2:\n%3")
									.arg(errorLine)
									.arg(errorColumn)
									.arg(errorStr));
			return false;
		}
		
		fillAccountList();
		unsavedChanges = false;
		
		return true;
	}
	return false;
}

bool MainWindow::saveFile(const QString &filename)
{
	QFile file(filename);
	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate))
	{
		const int IndentSize = 4;
		saveFileName = filename;
	
		QTextStream out(&file);
		domDocument.save(out, IndentSize);
		unsavedChanges = false;
		
		return true;
	}
	return false;
}

void MainWindow::newFile()
{
	domDocument = QDomDocument("qanac");
	domDocument.appendChild(domDocument.createElement("qanac"));
	fillAccountList();
	saveFileName.clear();
}

void MainWindow::load()
{
	suggestSaveIfRequierd();
	QString filename = QFileDialog::getOpenFileName(this, "", "", "Accounting (*.qanac)");
	if (filename != "")
		loadFile(filename);
}

void MainWindow::save()
{
	if (saveFileName.isEmpty())
	{
		QString filename = QFileDialog::getSaveFileName(this, "", "", "Accounting (*.qanac)");
		if (filename != "")
		{
			if (filename.lastIndexOf(".") < 0)
				filename += ".qanac";
			saveFile(filename);
		}
	}
	else
		saveFile(saveFileName);
}

void MainWindow::suggestSaveIfRequierd()
{
	if (unsavedChanges)
	{
		int ret = QMessageBox::warning(this, tr("Qanac"),
						tr("The current file has been modified.\n"
						"Do you want to save your changes?"),
						QMessageBox::Yes | QMessageBox::Default,
						QMessageBox::No);
		if (ret == QMessageBox::Yes)
			 save();
	}
}

void MainWindow::closeEvent ( QCloseEvent * event )
{
	suggestSaveIfRequierd();
	event->accept();
}

void MainWindow::newAccount()
{
	QDomElement e = domDocument.createElement("account");
	e.setAttribute("name", tr("unnamed"));
	domDocument.documentElement().appendChild(e);
	
	fillAccountList();
}

void MainWindow::deleteAccount()
{
	if (accountList->currentRow() >= 0)
	{
		domDocument.documentElement().removeChild(domElementForAccount[accountList->currentRow()]);
		fillAccountList();
	}
}

void MainWindow::renameAccount(QTableWidgetItem * item)
{
	domElementForAccount[accountList->row(item)].setAttribute("name", item->text());
	fillAccountList();
}

void MainWindow::newTransfer()
{
	if (accountList->currentRow() >= 0)
	{
		TransferDialog transferDialog(accountList->currentRow(), -1, accountList);
		int ret = transferDialog.exec();
		if (ret == QDialog::Rejected)
			return;
		
		if (!transferDialog.sourceEnabled->isChecked() && !transferDialog.destinationEnabled->isChecked())
			return;
		if (transferDialog.sourceEnabled->isChecked() && transferDialog.destinationEnabled->isChecked() && (transferDialog.source->currentText() == transferDialog.destination->currentText()))
			return;
		
		QDomElement e = domDocument.createElement("transfer");
		e.setAttribute("amount", transferDialog.amount->value());
		if (transferDialog.sourceEnabled->isChecked())
			e.setAttribute("source", transferDialog.source->currentText());
		if (transferDialog.destinationEnabled->isChecked())
			e.setAttribute("destination", transferDialog.destination->currentText());
		e.setAttribute("date", QDate::currentDate().toString("yyyyMMdd"));
		domDocument.documentElement().appendChild(e);
		
		int row = accountList->currentRow();
		fillAccountList();
		fillTransferList(row);
		accountList->setCurrentCell(row, 0);
	}
}

void MainWindow::deleteTransfer()
{
	if (transferList->currentRow() >= 0)
	{
		domDocument.documentElement().removeChild(domElementForTransfer[transferList->currentRow()]);
		
		int row = accountList->currentRow();
		fillAccountList();
		fillTransferList(row);
		accountList->setCurrentCell(row, 0);
	}
}

int MainWindow::computeBalanceForAccount(const QString &name)
{
	int balance = 0;
	QDomElement n = domDocument.documentElement().firstChildElement("transfer");
	while (!n.isNull())
	{
		if (n.attribute("source") == name)
		{
			if (n.attribute("destination") != name)
				balance -= n.attribute("amount").toInt();
		}
		else if (n.attribute("destination") == name)
		{
			balance += n.attribute("amount").toInt();
		}
		n = n.nextSiblingElement ("transfer");
	}
	return balance;
}

void MainWindow::fillAccountList()
{
	disconnect(accountList, SIGNAL(itemChanged(QTableWidgetItem *)), this, SLOT(renameAccount(QTableWidgetItem *)));
	
	domElementForAccount.clear();
	accountList->clear();
	domElementForTransfer.clear();
	transferList->clear();
	
	QDomElement n = domDocument.documentElement().firstChildElement("account");
	unsigned count = 0;
	while (!n.isNull())
	{
		n = n.nextSiblingElement ("account");
		count++;
	}
	
	accountList->setRowCount(count);
	
	n = domDocument.documentElement().firstChildElement("account");
	unsigned i = 0;
	int totalBalance = 0;
	while (!n.isNull())
	{
		QTableWidgetItem *newItem;
		
		newItem = new QTableWidgetItem(n.attribute("name"));
		newItem->setFlags(Qt::ItemIsEditable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		accountList->setItem(i, 0, newItem);
		
		// find amount
		int balance = computeBalanceForAccount(n.attribute("name"));
		totalBalance += balance;
		newItem = new QTableWidgetItem(QString("%1").arg(balance));
		if (balance < 0)
			newItem->setTextColor(Qt::red); 
		newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
		accountList->setItem(i, 1, newItem);
		
		domElementForAccount.push_back(n);
		n = n.nextSiblingElement ("account");
		i++;
	}
	
	if (totalBalance < 0)
		balance->setText(QString("<font color='red'>%1</font>").arg(totalBalance));
	else
		balance->setText(QString("%1").arg(totalBalance));
	
	connect(accountList, SIGNAL(itemChanged(QTableWidgetItem *)), this, SLOT(renameAccount(QTableWidgetItem *)));
	unsavedChanges = true;
}

void MainWindow::fillTransferList(int currentRow)
{
	domElementForTransfer.clear();
	
	if (currentRow < 0)
	{
		transferList->setRowCount(0);
		transferList->setColumnCount(0);
		return;
	}
	
	QString thisAccount = accountList->item(currentRow, 0)->text();
	
	// count number of element
	QDomElement n = domDocument.documentElement().firstChildElement("transfer");
	unsigned count = 0;
	while (!n.isNull())
	{
		if ((n.attribute("source") == thisAccount) || (n.attribute("destination") == thisAccount))
			count++;
		n = n.nextSiblingElement ("transfer");
	}
	
	if (count == 0)
	{
		transferList->setRowCount(0);
		transferList->setColumnCount(0);
		return;
	}
	
	transferList->setRowCount(count);
	transferList->setColumnCount(4);
	transferList->clear();
	
	// fill table
	n = domDocument.documentElement().firstChildElement("transfer");
	unsigned i = 0;
	while (!n.isNull())
	{
		QTableWidgetItem *newItem;
		
		if (n.hasAttribute("source") && n.attribute("source") == thisAccount)
		{
			QDate date = QDate::fromString(n.attribute("date"), "yyyyMMdd");
			newItem = new QTableWidgetItem(date.toString("d MMM yyyy"));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 0, newItem);
			
			newItem = new QTableWidgetItem(QString("%1").arg(n.attribute("amount")));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 2, newItem);
			
			if (n.hasAttribute("destination"))
				newItem = new QTableWidgetItem(QString(tr("to %1")).arg(n.attribute("destination")));
			else
				newItem = new QTableWidgetItem(QString(tr("as expense")));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 3, newItem);
			
			domElementForTransfer.push_back(n);
			i++;
		}
		
		if (n.hasAttribute("destination") && n.attribute("destination") == thisAccount)
		{
			QDate date = QDate::fromString(n.attribute("date"), "yyyyMMdd");
			newItem = new QTableWidgetItem(date.toString("d MMM yyyy"));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 0, newItem);
			
			newItem = new QTableWidgetItem(n.attribute("amount"));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 1, newItem);
			
			if (n.hasAttribute("source"))
				newItem = new QTableWidgetItem(QString(tr("from %1")).arg(n.attribute("source")));
			else
				newItem = new QTableWidgetItem(QString(tr("as income")));
			newItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
			transferList->setItem(i, 3, newItem);
			
			domElementForTransfer.push_back(n);
			i++;
		}
		
		n = n.nextSiblingElement ("transfer");
	}
}

void MainWindow::setupBudget()
{
	BudgetDialog budgetDialog(accountList, &domDocument);
	budgetDialog.exec();
	unsavedChanges = true;
}

void MainWindow::distributeBudget()
{
	QDomElement budgetElement = domDocument.documentElement().firstChildElement("budget");
	if (!budgetElement.isNull())
	{
		const QString &source = budgetElement.attribute("source");
		QDomElement n = budgetElement.firstChildElement("transfer");
		while (!n.isNull())
		{
			if (source != n.attribute("destination"))
			{
				QDomElement e = domDocument.createElement("transfer");
				e.setAttribute("amount", n.attribute("amount"));
				e.setAttribute("source", source);
				e.setAttribute("destination", n.attribute("destination"));
				e.setAttribute("date", QDate::currentDate().toString("yyyyMMdd"));
				domDocument.documentElement().appendChild(e);
			}
			n = n.nextSiblingElement ("transfer");
		}
		fillAccountList();
	}
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("Qanac"),
				tr("<h4 align=\"center\">Quick Analytic Accounting</h4>"
				"<h5 align=\"center\">Version 0.1</h5>"
				"<p>(c) 2006 Stephane Magnenat<br/>"
				"(stephane at magnenant dot net)</p>"
				"<p>This program is GPL, see license for details</p>"));
}
