/*

Qanac, the Quick Analytic Accounting
Copyright (C) 2006 Stephane Magnenat <stephane at magnenat dot net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#ifndef __MAINWINDOW_H
#define __MAINWINDOW_H

#include <QMainWindow>
#include <QDomDocument>
#include <QDomElement>
#include <QVector>
#include <QString>

class QLabel;
class QListWidget;
class QTableWidget;
class QPushButton;
class QAction;
class QToolBar;
class QTableWidgetItem;
class QCloseEvent;

class MainWindow : public QMainWindow
{
	Q_OBJECT

public:
	MainWindow();
	bool loadFile(const QString &filename);
	bool saveFile(const QString &filename);
/*
private slots:
	void open();
	void saveAs();
	void about();
*/
private slots:
	void newFile();
	void load();
	void save();
	void newAccount();
	void deleteAccount();
	void renameAccount(QTableWidgetItem * item);
	void newTransfer();
	void deleteTransfer();
	void fillTransferList(int currentRow);
	void setupBudget();
	void distributeBudget();
	void about();
	
protected:
	virtual void closeEvent ( QCloseEvent * event );

private:
	void createActions();
	void createMenus();
	void createToolBars();
	void createStatusBar();
	
	int computeBalanceForAccount(const QString &name);
	void fillAccountList();
	void suggestSaveIfRequierd();
	
	QVector<QDomElement> domElementForAccount;
	QVector<QDomElement> domElementForTransfer;
	QDomDocument domDocument;
	QString saveFileName;
	bool unsavedChanges;
	
	QTableWidget *accountList;
	QPushButton *newAccountButton;
	QPushButton *deleteAccountButton;
	QTableWidget *transferList;
	QPushButton *newTransferButton;
	QPushButton *deleteTransferButton;
	QLabel *balance;
	
	QMenu *fileMenu;
	QMenu *budgetMenu;
	QMenu *helpMenu;
	
	QToolBar *budgetToolBar;
	
	QAction *newAct;
	QAction *openAct;
	QAction *saveAct;
	QAction *quitAct;
	
	QAction *setupBudgetAct;
	QAction *distributeBudgetAct;
	
	QAction *aboutAct;
	QAction *aboutQtAct;
};

#endif
