/*

Qanac, the Quick Analytic Accounting
Copyright (C) 2006 Stephane Magnenat <stephane at magnenat dot net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "transferdialog.h"
#include <QDoubleSpinBox>
#include <QCheckBox>
#include <QComboBox>
#include <QTableWidget>
#include <QPushButton>
#include <QVBoxLayout>


TransferDialog::TransferDialog(int sourceId, int destId, const QTableWidget *accounts)
{
	QVBoxLayout *layout = new QVBoxLayout(this);
	
	amount = new QDoubleSpinBox(this);
	amount->setRange(0, 1000000000);
	
	sourceEnabled = new QCheckBox(this);
	sourceEnabled->setChecked(sourceId >= 0);
	sourceEnabled->setText(tr("source account"));
	source = new QComboBox(this);
	source->setEnabled(sourceId >= 0);
	
	destinationEnabled = new QCheckBox(this);
	destinationEnabled->setChecked(destId >= 0);
	destinationEnabled->setText(tr("destination account"));
	destination = new QComboBox(this);
	destination->setEnabled(destId >= 0);
	
	for (int i = 0; i < accounts->rowCount(); i++)
	{
		const QString &accountName = accounts->item(i, 0)->text();
		source->addItem(accountName);
		destination->addItem(accountName);
	}
	if (sourceId >= 0)
		source->setCurrentIndex(sourceId);
	if (destId >= 0)
		destination->setCurrentIndex(destId);
	
	connect(sourceEnabled, SIGNAL(toggled(bool)), source, SLOT(setEnabled(bool)));
	connect(destinationEnabled, SIGNAL(toggled(bool)), destination, SLOT(setEnabled(bool)));
	
	layout->addWidget(amount);
	layout->addWidget(sourceEnabled);
	layout->addWidget(source);
	layout->addWidget(destinationEnabled);
	layout->addWidget(destination);
	
	QPushButton *ok = new QPushButton(tr("Ok"));
	connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
	layout->addWidget(ok);
	
	setWindowTitle(tr("Transfer"));
}
